/**
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    Armar4::api
* @author     Nikolaus Vahrenkamp
* @date       2012 Nikolaus Vahrenkamp
* @copyright  http://www.gnu.org/licenses/gpl.txt
*             GNU General Public License
*/


#ifndef ARMARX_COMPONENT_Armar4Api_Armar4Context_H
#define ARMARX_COMPONENT_Armar4Api_Armar4Context_H

#include <Core/core/Component.h>
#include <Core/core/system/ImportExportComponent.h>
#include <Core/statechart/StatechartContext.h>
#include <Core/robotstate/remote/RemoteRobot.h>
#include <Core/interface/cpp/units/KinematicUnitInterface.h>

#include <Core/units/KinematicUnitObserver.h>
//#include <VirtualRobot/VirtualRobot.h>
#include <IceUtil/Time.h>

namespace armarx
{

    // ****************************************************************
    // Component and context
    // ****************************************************************

    struct RobotStatechartContextProperties : ComponentPropertyDefinitions
    {
        RobotStatechartContextProperties(std::string prefix):
            ComponentPropertyDefinitions(prefix)
        {
            defineRequiredProperty<std::string>("KinematicUnitName", "Name of the kinematic unit that should be used");
            defineRequiredProperty<std::string>("KinematicUnitObserverName", "Name of the kinematic unit observer that should be used");

        }
    };

    /**
      * @class HumanoidRobotStatechartContext is the implementation of the StatechartContext
      * for a HumanoidRobot
      */
    class ARMARXCOMPONENT_IMPORT_EXPORT RobotStatechartContext :
        virtual public StatechartContext
    {
    public:
        // inherited from Component
        virtual std::string getDefaultName() { return "RobotStatechartContext"; };
        virtual void onInitStatechart();
        virtual void onConnectStatechart();

        // todo:read access should only be allowed via const getters ?!
        //const VirtualRobot::RobotPtr getRobot();
    //private:


        /**
         * @see PropertyUser::createPropertyDefinitions()
         */
        virtual PropertyDefinitionsPtr createPropertyDefinitions();

        std::string getKinematicUnitObserverName() { return kinematicUnitObserverName; }

        //! Prx for the RobotState
		RobotStateComponentInterfacePrx robotStateComponent;
		KinematicUnitInterfacePrx kinematicUnitPrx;
        KinematicUnitObserverInterfacePrx kinematicUnitObserverPrx;

        //SystemObserverInterfacePrx	systemObserver; // already defined in StatechartContext
        VirtualRobot::RobotPtr remoteRobot;
    private:
        std::string kinematicUnitObserverName;
    };
}

#endif
